# MailParser.pm
#
# A module for parsing "simple mails" and GnuPG signed mails.
#
# author:  Max-Gerd Retzlaff <m.retzlaff@gmx.net>
#
#          MIME::Parser stuff in SignedMIMEMail::get_content()
#             by Reyk Floeter <reyk@synack.de>
#
# date:    $Date: 2002/03/07 01:34:47 $
# version: $Revision: 1.6 $
#
# This application is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version. 
#
# This application is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this library; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# 02111 USA.  

package MailParser;

use HelpfulFunctions;
# use MIME::Parser;
use Parser; # patched MIME::Parser in current directory
use GnuPGverify;

my $SimpleMailAllowed = 'no';

sub new {
    my ($pkg, $text) = @_;
    return (bless \$text, $pkg);
}

sub AllowSimpleMail {
    $SimpleMailAllowed = 'yes';
}

sub ForbidSimpleMail {
    $SimpleMailAllowed = 'no';
}

sub autodetectingnew {
    #my ($pkg, $text) = @_;
    my $pkg = $_[0];
    my $text = $_[1];
    my %sections = %{$_[2]};

    my $content_type = get_header_field(\$text, 'Content-Type');
    if ( $content_type =~ /text\/plain;/ or $content_type eq '') {
	# $content_type eq '' added because of Mail::Mailer-limitations (DIGEST)
	my ($command, $alias, undef, undef) = MailParser::parse_subject(\$text);
	
	my $section = undef;
	foreach my $valid_section (keys %sections) {
	    foreach my $valid_alias (@{$sections{$valid_section}->{aliases}} ) {
		if ($alias eq $valid_alias) {
		    $section = $valid_section;
		}
	    }
	}
	
	if ( $SimpleMailAllowed eq 'yes' or
	     $sections{$section}->{AllowSimpleMail} =~ /^AD[MD]$/ or
	     $command eq 'SENDDIGEST' or
	     $command eq 'DIGEST' ) {
	    return ( new('SimpleMail', $text) );
	} else {
	    mdie(0, "ERROR: SimpleMail is not allowed.\n");
	}
    } elsif ( $content_type =~ /multipart\/signed;/ ) {
	# SignedMIMEMail
	return ( new('SignedMIMEMail', $text) );
    } else {
	mdie(0, "ERROR: Can't parse Content-Type: $content_type.\n");
    }
}

sub get_header {
    my $text = ${$_[0]}; #no shift
    my ($header, undef) = split(/\n\n/, $text, 2);
    return $header;
}
# using shift you could change the mailtext in get_header():
#sub get_header {
#    my $pkg = shift;
#    my ($header, $body) = split(/\n\n/, $$pkg, 2);
#    $$pkg = 'No more lemmings!';
#    return $header;
#}

sub get_header_field {
    my $text = ${ $_[0] };
    my $field = $_[1];

    my @header = split('\n', get_header(\$text) );
    # my $hit = 0;
    my $value = '';
    foreach my $line (@header) {
	if ( $line =~ /^$field:\s(.*?)$/i ) {
	    # $hit = 1;
	    $value = $1;
	}
    }
    return $value;
}

sub replace_header_field {
    my ($text_r, $field, $new_value, $stamp_mode) = @_;

    my ($header, $body) = split(/\n\n/, $$text_r, 2);
    my @header = split('\n', $header);
    
    my $inserted = 0;
    for(my $i=0; $i <= $#header; $i++) {
	if ( @header[$i] =~ /^$field:\s(.*?)$/i ) {
	    my $value = $1;

	    if ($value ne $new_value and !defined($stamp_mode) ) {
		@header[$i] = "X-Original-$field: $value";
		if ( $inserted == 0 ) {
		    @header[$i] .= "\n$field: $new_value";
		    $inserted = 1;
		}
	    } else {
		$inserted = 2;
	    }
	}	
    }	
    
    if ( $inserted == 0 ) {
	push(@header, "$field: $new_value");
	$inserted = 1;
    }

    $header = join("\n", @header);
    $$text_r = "$header\n\n$body";

    return 0;
}

# became obsolete, use replace_header_field instead
#sub replace_subject_and_mid { # CHANGES SUPPLIED TEXT (that is not nice...)
#    my ($text_r, $command, $alias, $mid, $title) = @_;
#
#    my $new_subject = "ADD: $alias: $title";
#    my $current_subject = get_header_field($text_r, 'Subject' );
#    if ($new_subject ne $current_subject ) {
#	$$text_r =~ s/(\n)(Subject:\s.*?\n)/$1X-Original-$2/gi;
#	$$text_r =~ s/(\n\n)/\nSubject: $new_subject$1/; # not s///g!
#    }
#
#    if ( $mid ne get_mid($text_r) ) {
#	$$text_r =~ s/(\n)(Message-ID:\s.*?\n)/$1X-Original-$2/gi;
#	$mid =~ s/-at-/\@/;
#	$$text_r =~ s/(\n\n)/\nMessage-ID: <$mid>$1/; # not s///g!
#    }
#
#    return 0;
#}

sub get_sender {
    my $text = ${ $_[0] };

    my $from = get_header_field(\$text, 'From' );

    if ($from eq '') { # no "From:" at all
	return 'anonymous';
    } elsif ( $from =~ m/(.*?)\s*?\<(.*?)\>/ ) { # John Doe <jdoe@no.where>
	return "<a href=\"mailto:$2\">$1</a>";
    } elsif ( $from =~ m/(.*?)\s*?\((.*?)\)/ ) { # jdoe@no.where (John Doe)
	return "<a href=\"mailto:$1\">$2</a>";
    } elsif ( $from =~ m/([^\s]*\@[^\s]*)/ ) { # blabla jdoe@no.where blabla
	return "<a href=\"mailto:$1\">$1</a>";
    } else { # can't recognise format, just returning
	return $from;
    }
}

sub get_mid {
    my $text = ${ $_[0] };

    my $mid = get_header_field(\$text, 'Message-ID' );
    $mid =~ s/<(.*)>/$1/;
    $mid =~ s/\@/-at-/;
    return $mid;
}

# obsolete -> now using parse_subject()
#
#sub get_type {
#    my $text = ${ $_[0] };
#
#    my $subject = get_header_field(\$text, 'Subject' );
#    (my $type = $subject )  =~ s/^(?:\[.*?\]\s)?(\w*):\s.*/$1/;
#    return $type;
#}
#
#sub get_title {
#    my $text = ${ $_[0] };
#
#    my $subject = get_header_field(\$text, 'Subject' );
#    (my $title = $subject ) =~ s/^(?:\[.*?\]\s)?\w*:\s(.*)/$1/;
#    return $title;
#}

sub parse_subject {
    my $text = ${ $_[0] };

    my $subject = get_header_field(\$text, 'Subject' );

    if ( $subject =~ m/^(?:Re:\s*)?(?:\[[^\[]*?\]\s+)?(?:(\w*):\s+)?(\w*):(?:\s+\[([^\[]*?)\])?\s+(.*)/ ) {
	# $1: command, $2: alias, $3: mid, $4 title

	my ($command, $alias, $mid, $title) = ( uc($1), uc($2), $3, $4);
	if ( !defined($command) ){
	    $command = 'ADD';
	}
	if ( $command eq 'UPDATE' or
	     $command eq 'UPGRADE' or
	     $command eq 'REPLACE' or
	     $command eq 'REMOVE' or
	     $command eq 'DELETE' or
	     $command eq 'SENDDIGEST' or
	     $command =~ /^SEND_/ ) {
	    if ( !defined($mid) ) {
		mdie(0, "ERROR: You have to supply a message ID if you invoke command $command.\n");
	    } 
	    $mid =~ s/\@/-at-/;
	    return ($command, $alias, $mid, $title);
	} else {
	    return ($command, $alias, '', $title);
	}
    } else {
	mdie(0, "ERROR: Can't parse Subject: $subject.\n");
    }
}

my $ExpectMagicString = 'yes';

sub ExpectMagicString {
    $ExpectMagicString = 'yes';
}

sub DoNotExpectMagicString {
    $ExpectMagicString = 'no';
}

sub magic_string_test {
    $text_r = shift;

    if ( $ExpectMagicString eq 'yes' ) {
	if ( $$text_r =~ /^\[C3H-POST\]\n/i ) {
	    $$text_r =~ s/^\[C3H-POST\]\n+//i;
	} else {
	    mdie(0, "ERROR: Magic string \[C3H-POST\] not found.\n");
	}
    }
}

    
##################################################
package SimpleMail;
@ISA = ('MailParser'); # inherits from MailParser

sub get_content {
    my $text = ${$_[0]}; #no shift;

    my (undef, $content) =  split/\n\n/, $text, 2;

    MailParser::magic_string_test(\$content);

    return ($content, 'SimpleMail');
}

##################################################
package SignedMIMEMail;
@ISA = ('MailParser'); # inherits from MailParser

sub get_content {
    my $text = ${$_[0]}; #no shift;  
    my %sections = %{$_[1]};
    
    my $parser = new MIME::Parser;
    # dirty hack: decode_body() requires a patched MIME::Parser
    $parser->decode_body(0);
    $parser->output_under("/tmp");
    $parser->extract_nested_messages(0);
    my $entity = $parser->parse_data($text);
    mdie(0, "ERROR: No RFC822 signature found.\n") if (!$entity->is_multipart());
    my $content_part = $entity->parts(0)->bodyhandle->as_string();
    my $gpg_signature = $entity->parts(1)->as_string();
    #my $alias = MailParser::get_mail_type(\$text);
    my (undef, $alias, undef, undef) = MailParser::parse_subject(\$text);

    my (undef, $sign_ID) = GnuPGverify->check_signature( $alias, $content_part, $gpg_signature, \%sections);

    # Isn't it stupid to create a new MIME::Parser object,
    # only to strip of the leading Content-*-lines? Uhm, later...
    # No!, this one does not use $parser->decode_body(0);
    my $parser2 = new MIME::Parser;
    my $entity2 = $parser2->parse_data($content_part);
    # Remove any mail signature ("^--\s" stuff) within 15 lines of the end
    $entity2->remove_sig(15);
    my $content = $entity2->bodyhandle->as_string();

    MailParser::magic_string_test(\$content);

    return ($content, $sign_ID);
}

return 1; # Last executing statement in file must be
          # non-zero, to indicate succesful loading
